@@ -24,8 +24,8 @@ template <typename DELETER>
 class db_copy_mgr_t
 {
 public:
-    explicit db_copy_mgr_t(std::shared_ptr<db_copy_thread_t> const &processor)
-    : m_processor(processor)
+    explicit db_copy_mgr_t(std::shared_ptr<db_copy_thread_t> processor)
+    : m_processor(std::move(processor))
     {}
 
     /**
@@ -43,6 +43,13 @@ public:
 
             m_current = std::make_unique<db_cmd_copy_delete_t<DELETER>>(table);
         }
+        m_committed = m_current->buffer.size();
+    }
+
+    void rollback_line()
+    {
+        assert(m_current);
+        m_current->buffer.resize(m_committed);
     }
 
     /**
@@ -333,10 +340,10 @@ private:
         for (char const *c = s; *c; ++c) {
             switch (*c) {
             case '"':
-                m_current->buffer += "\\\\\"";
+                m_current->buffer += R"(\\")";
                 break;
             case '\\':
-                m_current->buffer += "\\\\\\\\";
+                m_current->buffer += R"(\\\\)";
                 break;
             case '\n':
                 m_current->buffer += "\\n";
@@ -356,6 +363,7 @@ private:
 
     std::shared_ptr<db_copy_thread_t> m_processor;
     std::unique_ptr<db_cmd_copy_delete_t<DELETER>> m_current;
+    std::size_t m_committed = 0;
 };
 
 #endif // OSM2PGSQL_DB_COPY_MGR_HPP
