@@ -23,7 +23,7 @@ struct column_type_lookup
     table_column_type type;
 };
 
-static std::array<column_type_lookup, 25> const column_types = {
+static std::array<column_type_lookup, 26> const column_types = {
     {{"text", table_column_type::text},
      {"boolean", table_column_type::boolean},
      {"bool", table_column_type::boolean},
@@ -46,6 +46,7 @@ static std::array<column_type_lookup, 25> const column_types = {
      {"multipoint", table_column_type::multipoint},
      {"multilinestring", table_column_type::multilinestring},
      {"multipolygon", table_column_type::multipolygon},
+     {"geometrycollection", table_column_type::geometrycollection},
      {"area", table_column_type::area},
      {"id_type", table_column_type::id_type},
      {"id_num", table_column_type::id_num}}};
@@ -53,6 +54,8 @@ static std::array<column_type_lookup, 25> const column_types = {
 static table_column_type
 get_column_type_from_string(std::string const &type)
 {
+    // Because it doesn't work with MSVC:
+    // NOLINTNEXTLINE(llvm-qualified-auto,readability-qualified-auto)
     auto const it =
         std::find_if(std::begin(column_types), std::end(column_types),
                      [&type](column_type_lookup name_type) {
@@ -80,9 +83,9 @@ static std::string lowercase(std::string const &str)
 
 flex_table_column_t::flex_table_column_t(std::string name,
                                          std::string const &type,
-                                         std::string const &sql_type)
+                                         std::string sql_type)
 : m_name(std::move(name)), m_type_name(lowercase(type)),
-  m_sql_type(sql_type),
+  m_sql_type(std::move(sql_type)),
   m_type(get_column_type_from_string(m_type_name))
 {}
 
@@ -154,6 +157,8 @@ std::string flex_table_column_t::sql_type_name() const
         return "Geometry(MULTILINESTRING, {})"_format(m_srid);
     case table_column_type::multipolygon:
         return "Geometry(MULTIPOLYGON, {})"_format(m_srid);
+    case table_column_type::geometrycollection:
+        return "Geometry(GEOMETRYCOLLECTION, {})"_format(m_srid);
     case table_column_type::area:
         return "real";
     case table_column_type::id_type:
@@ -181,5 +186,5 @@ std::string flex_table_column_t::sql_modifiers() const
 
 std::string flex_table_column_t::sql_create() const
 {
-    return "\"{}\" {} {},"_format(m_name, sql_type_name(), sql_modifiers());
+    return R"("{}" {} {},)"_format(m_name, sql_type_name(), sql_modifiers());
 }
