@@ -73,12 +73,14 @@ flex_table_column_t &flex_table_t::add_column(std::string const &name,
            (m_columns.size() == 1 &&
             m_columns[0].type() == table_column_type::id_type));
 
-    m_columns.emplace_back(name, type, sql_type);
-    auto &column = m_columns.back();
+    auto &column = m_columns.emplace_back(name, type, sql_type);
 
     if (column.is_geometry_column()) {
-        m_geom_column = m_columns.size() - 1;
-        column.set_not_null();
+        if (m_geom_column == std::numeric_limits<std::size_t>::max()) {
+            m_geom_column = m_columns.size() - 1;
+        } else {
+            m_has_multiple_geom_columns = true;
+        }
     }
 
     return column;
@@ -159,6 +161,30 @@ void table_connection_t::connect(std::string const &conninfo)
     m_db_connection->exec("SET synchronous_commit = off");
 }
 
+static void
+enable_check_trigger(pg_conn_t *db_connection, flex_table_t const &table)
+{
+    std::string checks;
+
+    for (auto const &column : table) {
+        if (column.is_geometry_column() && column.needs_isvalid()) {
+            checks.append(
+                R"((NEW."{0}" IS NULL OR ST_IsValid(NEW."{0}")) AND )"_format(
+                    column.name()));
+        }
+    }
+
+    if (checks.empty()) {
+        return;
+    }
+
+    // remove last " AND "
+    checks.resize(checks.size() - 5);
+
+    create_geom_check_trigger(db_connection, table.schema(), table.name(),
+                              checks);
+}
+
 void table_connection_t::start(bool append)
 {
     assert(m_db_connection);
@@ -181,12 +207,7 @@ void table_connection_t::start(bool append)
                                       : flex_table_t::table_type::permanent,
             table().full_name()));
 
-        if (table().has_geom_column() &&
-            table().geom_column().needs_isvalid()) {
-            create_geom_check_trigger(m_db_connection.get(), table().schema(),
-                                      table().name(),
-                                      table().geom_column().name());
-        }
+        enable_check_trigger(m_db_connection.get(), table());
     }
 
     prepare();
@@ -247,14 +268,12 @@ void table_connection_t::stop(bool updateable, bool append)
         m_db_connection->exec(sql);
 
         m_db_connection->exec("DROP TABLE {}"_format(table().full_name()));
-        m_db_connection->exec("ALTER TABLE {} RENAME TO \"{}\""_format(
+        m_db_connection->exec(R"(ALTER TABLE {} RENAME TO "{}")"_format(
             table().full_tmp_name(), table().name()));
         m_id_index_created = false;
 
-        if (updateable && table().geom_column().needs_isvalid()) {
-            create_geom_check_trigger(m_db_connection.get(), table().schema(),
-                                      table().name(),
-                                      table().geom_column().name());
+        if (updateable) {
+            enable_check_trigger(m_db_connection.get(), table());
         }
     }
 
@@ -263,7 +282,7 @@ void table_connection_t::stop(bool updateable, bool append)
 
         // Use fillfactor 100 for un-updateable imports
         m_db_connection->exec(
-            "CREATE INDEX ON {} USING GIST (\"{}\") {} {}"_format(
+            R"(CREATE INDEX ON {} USING GIST ("{}") {} {})"_format(
                 table().full_name(), table().geom_column().name(),
                 (updateable ? "" : "WITH (fillfactor = 100)"),
                 tablespace_clause(table().index_tablespace())));
@@ -274,7 +293,7 @@ void table_connection_t::stop(bool updateable, bool append)
     }
 
     log_info("Analyzing table '{}'...", table().name());
-    analyze_table(*m_db_connection, table().schema(), table().name());
+    analyze();
 
     teardown();
 }
@@ -287,6 +306,11 @@ void table_connection_t::prepare()
     }
 }
 
+void table_connection_t::analyze()
+{
+    analyze_table(*m_db_connection, table().schema(), table().name());
+}
+
 void table_connection_t::create_id_index()
 {
     if (m_id_index_created) {
