@@ -12,8 +12,8 @@
 
 #include "db-copy-mgr.hpp"
 #include "flex-table-column.hpp"
-#include "osmium-builder.hpp"
 #include "pgsql.hpp"
+#include "reprojection.hpp"
 #include "thread-pool.hpp"
 
 #include <osmium/osm/item_type.hpp>
@@ -110,7 +110,7 @@ public:
         return m_geom_column != std::numeric_limits<std::size_t>::max();
     }
 
-    // XXX should we allow several geometry columns?
+    /// Get the (first, if there are multiple) geometry column.
     flex_table_column_t const &geom_column() const noexcept
     {
         assert(has_geom_column());
@@ -191,6 +191,11 @@ public:
     std::string full_name() const;
     std::string full_tmp_name() const;
 
+    bool has_multiple_geom_columns() const noexcept
+    {
+        return m_has_multiple_geom_columns;
+    }
+
 private:
     /// The name of the table
     std::string m_name;
@@ -213,7 +218,10 @@ private:
      */
     std::vector<flex_table_column_t> m_columns;
 
-    /// Index of the geometry column in m_columns. Default means no geometry.
+    /**
+     * Index of the (first) geometry column in m_columns. Default means no
+     * geometry column.
+     */
     std::size_t m_geom_column = std::numeric_limits<std::size_t>::max();
 
     /**
@@ -225,6 +233,9 @@ private:
     /// Cluster the table by geometry.
     bool m_cluster_by_geom = true;
 
+    /// Does this table have more than one geometry column?
+    bool m_has_multiple_geom_columns = false;
+
 }; // class flex_table_t
 
 class table_connection_t
@@ -232,7 +243,7 @@ class table_connection_t
 public:
     table_connection_t(flex_table_t *table,
                        std::shared_ptr<db_copy_thread_t> const &copy_thread)
-    : m_builder(reprojection::create_projection(table->srid())), m_table(table),
+    : m_proj(reprojection::create_projection(table->srid())), m_table(table),
       m_target(std::make_shared<db_target_descr_t>(
           table->name(), table->id_column_names(),
           table->build_sql_column_list())),
@@ -253,6 +264,8 @@ public:
 
     void prepare();
 
+    void analyze();
+
     void create_id_index();
 
     pg_result_t get_geom_by_id(osmium::item_type type, osmid_t id) const;
@@ -268,7 +281,11 @@ public:
 
     void delete_rows_with(osmium::item_type type, osmid_t id);
 
-    geom::osmium_builder_t *get_builder() { return &m_builder; }
+    reprojection const &proj() const noexcept
+    {
+        assert(m_proj);
+        return *m_proj;
+    }
 
     void task_set(std::future<std::chrono::milliseconds> &&future)
     {
@@ -278,7 +295,7 @@ public:
     void task_wait();
 
 private:
-    geom::osmium_builder_t m_builder;
+    std::shared_ptr<reprojection> m_proj;
 
     flex_table_t *m_table;
 
