@@ -8,6 +8,7 @@
  */
 
 #include <algorithm>
+#include <array>
 #include <cstring>
 #include <stdexcept>
 
@@ -307,8 +308,13 @@ void gazetteer_style_t::process_tags(osmium::OSMObject const &o)
         char const *const v = item.value();
 
         if (std::strcmp(k, "admin_level") == 0) {
-            m_admin_level = std::atoi(v);
-            if (m_admin_level <= 0 || m_admin_level > MAX_ADMINLEVEL) {
+            char *endp = nullptr;
+            errno = 0;
+            auto const parsed = std::strtol(v, &endp, 10);
+            if (errno == 0 && *endp == '\0' && parsed > 0 &&
+                parsed < MAX_ADMINLEVEL) {
+                m_admin_level = static_cast<int>(parsed);
+            } else {
                 m_admin_level = MAX_ADMINLEVEL;
             }
             continue;
@@ -337,7 +343,7 @@ void gazetteer_style_t::process_tags(osmium::OSMObject const &o)
             }
         }
 
-        if (flag & (SF_NAME | SF_REF)) {
+        if (flag & static_cast<flag_t>(SF_NAME | SF_REF)) {
             m_names.emplace_back(k, v);
             if (flag & SF_NAME) {
                 is_named = true;
@@ -458,113 +464,113 @@ void gazetteer_style_t::filter_main_tags(bool is_named,
 
 void gazetteer_style_t::copy_out(osmium::OSMObject const &o,
                                  std::string const &geom,
-                                 copy_mgr_t &buffer) const
+                                 copy_mgr_t *buffer) const
 {
     for (auto const &tag : m_main) {
-        buffer.prepare();
+        buffer->prepare();
         // osm_id
-        buffer.add_column(o.id());
+        buffer->add_column(o.id());
         // osm_type
-        char const osm_type[2] = {
+        std::array<char const, 2> const osm_type = {
             (char)toupper(osmium::item_type_to_char(o.type())), '\0'};
-        buffer.add_column(osm_type);
+        buffer->add_column(osm_type.data());
         // class
-        buffer.add_column(std::get<0>(tag));
+        buffer->add_column(std::get<0>(tag));
         // type
-        buffer.add_column(std::get<1>(tag));
+        buffer->add_column(std::get<1>(tag));
         // names
         if (std::get<2>(tag) & SF_MAIN_NAMED_KEY) {
             DomainMatcher m{std::get<0>(tag)};
-            buffer.new_hash();
+            buffer->new_hash();
             for (auto const &t : o.tags()) {
                 char const *const k = m(t);
                 if (k) {
-                    buffer.add_hash_elem(k, t.value());
+                    buffer->add_hash_elem(k, t.value());
                 }
             }
-            buffer.finish_hash();
+            buffer->finish_hash();
         } else {
             bool first = true;
             // operator will be ignored on anything but these classes
             if (m_operator && (std::get<2>(tag) & SF_MAIN_OPERATOR)) {
-                buffer.new_hash();
-                buffer.add_hash_elem("operator", m_operator);
+                buffer->new_hash();
+                buffer->add_hash_elem("operator", m_operator);
                 first = false;
             }
-            for (auto const &entry : m_names) {
+            for (auto const &[key, value] : m_names) {
                 if (first) {
-                    buffer.new_hash();
+                    buffer->new_hash();
                     first = false;
                 }
 
-                buffer.add_hash_elem(entry.first, entry.second);
+                buffer->add_hash_elem(key, value);
             }
 
             if (first) {
-                buffer.add_null_column();
+                buffer->add_null_column();
             } else {
-                buffer.finish_hash();
+                buffer->finish_hash();
             }
         }
         // admin_level
-        buffer.add_column(m_admin_level);
+        buffer->add_column(m_admin_level);
         // address
         if (m_address.empty()) {
-            buffer.add_null_column();
+            buffer->add_null_column();
         } else {
-            buffer.new_hash();
-            for (auto const &a : m_address) {
-                if (std::strcmp(a.first, "tiger:county") == 0) {
+            buffer->new_hash();
+            for (auto const &[key, value] : m_address) {
+                if (std::strcmp(key, "tiger:county") == 0) {
                     std::string term;
-                    auto const *const end = std::strchr(a.second, ',');
+                    auto const *const end = std::strchr(value, ',');
                     if (end) {
                         auto const len =
-                            (std::string::size_type)(end - a.second);
-                        term = std::string(a.second, len);
+                            (std::string::size_type)(end - value);
+                        term = std::string(value, len);
                     } else {
-                        term = a.second;
+                        term = value;
                     }
                     term += " county";
-                    buffer.add_hash_elem(a.first, term);
+                    buffer->add_hash_elem(key, term);
                 } else {
-                    buffer.add_hash_elem(a.first, a.second);
+                    buffer->add_hash_elem(key, value);
                 }
             }
-            buffer.finish_hash();
+            buffer->finish_hash();
         }
         // extra tags
         if (m_extra.empty() && m_metadata_fields.none()) {
-            buffer.add_null_column();
+            buffer->add_null_column();
         } else {
-            buffer.new_hash();
-            for (auto const &entry : m_extra) {
-                buffer.add_hash_elem(entry.first, entry.second);
+            buffer->new_hash();
+            for (auto const &[key, value] : m_extra) {
+                buffer->add_hash_elem(key, value);
             }
             if (m_metadata_fields.version() && o.version()) {
-                buffer.add_hstore_num_noescape<osmium::object_version_type>(
+                buffer->add_hstore_num_noescape<osmium::object_version_type>(
                     "osm_version", o.version());
             }
             if (m_metadata_fields.uid() && o.uid()) {
-                buffer.add_hstore_num_noescape<osmium::user_id_type>("osm_uid",
-                                                                     o.uid());
+                buffer->add_hstore_num_noescape<osmium::user_id_type>("osm_uid",
+                                                                      o.uid());
             }
             if (m_metadata_fields.user() && o.user() && *(o.user()) != '\0') {
-                buffer.add_hash_elem("osm_user", o.user());
+                buffer->add_hash_elem("osm_user", o.user());
             }
             if (m_metadata_fields.changeset() && o.changeset()) {
-                buffer.add_hstore_num_noescape<osmium::changeset_id_type>(
+                buffer->add_hstore_num_noescape<osmium::changeset_id_type>(
                     "osm_changeset", o.changeset());
             }
             if (m_metadata_fields.timestamp() && o.timestamp()) {
                 std::string timestamp = o.timestamp().to_iso();
-                buffer.add_hash_elem_noescape("osm_timestamp",
-                                              timestamp.c_str());
+                buffer->add_hash_elem_noescape("osm_timestamp",
+                                               timestamp.c_str());
             }
-            buffer.finish_hash();
+            buffer->finish_hash();
         }
         // add the geometry - encoding it to hex along the way
-        buffer.add_hex_geom(geom);
+        buffer->add_hex_geom(geom);
 
-        buffer.finish_line();
+        buffer->finish_line();
     }
 }
