@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2022 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -15,14 +15,13 @@
  * emit the final geometry-enabled output formats
 */
 
-#include <stdexcept>
-#include <unordered_map>
-
 #include <cassert>
-#include <cstdio>
 #include <cstdlib>
-#include <cstring>
-#include <functional>
+#include <memory>
+#include <stdexcept>
+#include <string>
+#include <unordered_map>
+#include <utility>
 
 #include <osmium/builder/osm_object_builder.hpp>
 #include <osmium/memory/buffer.hpp>
@@ -31,8 +30,8 @@
 #include "format.hpp"
 #include "logging.hpp"
 #include "middle-pgsql.hpp"
+#include "node-locations.hpp"
 #include "node-persistent-cache.hpp"
-#include "node-ram-cache.hpp"
 #include "options.hpp"
 #include "osmtypes.hpp"
 #include "pgsql-helper.hpp"
@@ -48,7 +47,7 @@ static std::string build_sql(options_t const &options, char const *templ)
         templ, fmt::arg("prefix", options.prefix),
         fmt::arg("schema", options.middle_dbschema.empty()
                                ? ""
-                               : (options.middle_dbschema + ".")),
+                               : ("\"" + options.middle_dbschema + "\".")),
         fmt::arg("unlogged", options.droptemp ? "UNLOGGED" : ""),
         fmt::arg("using_tablespace", using_tablespace),
         fmt::arg("data_tablespace", tablespace_clause(options.tblsslim_data)),
@@ -88,7 +87,7 @@ void middle_pgsql_t::table_desc::drop_table(
     auto const qual_name = qualified_name(schema(), name());
     db_connection.exec("DROP TABLE IF EXISTS {}"_format(qual_name));
 
-    log_info("Done postprocessing on table '{}' in {}", name(),
+    log_info("Table '{}' dropped in {}", name(),
              util::human_readable_duration(timer.stop()));
 }
 
@@ -98,17 +97,12 @@ void middle_pgsql_t::table_desc::build_index(std::string const &conninfo) const
         return;
     }
 
-    util::timer_t timer;
-
     // Use a temporary connection here because we might run in a separate
     // thread context.
     pg_conn_t db_connection{conninfo};
 
     log_info("Building index on table '{}'", name());
     db_connection.exec(m_create_fw_dep_indexes);
-
-    log_info("Done postprocessing on table '{}' in {}", name(),
-             util::human_readable_duration(timer.stop()));
 }
 
 namespace {
@@ -148,7 +142,7 @@ inline char const *decode_upto(char const *src, char *dst)
 
 template <typename T>
 void pgsql_parse_tags(char const *string, osmium::memory::Buffer *buffer,
-                      T &obuilder)
+                      T *obuilder)
 {
     if (*string++ != '{') {
         return;
@@ -156,7 +150,7 @@ void pgsql_parse_tags(char const *string, osmium::memory::Buffer *buffer,
 
     char key[1024];
     char val[1024];
-    osmium::builder::TagListBuilder builder{*buffer, &obuilder};
+    osmium::builder::TagListBuilder builder{*buffer, obuilder};
 
     while (*string != '}') {
         string = decode_upto(string, key);
@@ -172,14 +166,14 @@ void pgsql_parse_tags(char const *string, osmium::memory::Buffer *buffer,
 }
 
 void pgsql_parse_members(char const *string, osmium::memory::Buffer *buffer,
-                         osmium::builder::RelationBuilder &obuilder)
+                         osmium::builder::RelationBuilder *obuilder)
 {
     if (*string++ != '{') {
         return;
     }
 
     char role[1024];
-    osmium::builder::RelationMemberListBuilder builder{*buffer, &obuilder};
+    osmium::builder::RelationMemberListBuilder builder{*buffer, obuilder};
 
     while (*string != '}') {
         char type = string[0];
@@ -196,10 +190,10 @@ void pgsql_parse_members(char const *string, osmium::memory::Buffer *buffer,
 }
 
 void pgsql_parse_nodes(char const *string, osmium::memory::Buffer *buffer,
-                       osmium::builder::WayBuilder &builder)
+                       osmium::builder::WayBuilder *obuilder)
 {
     if (*string++ == '{') {
-        osmium::builder::WayNodeListBuilder wnl_builder{*buffer, &builder};
+        osmium::builder::WayNodeListBuilder wnl_builder{*buffer, obuilder};
         while (*string != '}') {
             char *ptr = nullptr;
             wnl_builder.add_node_ref(std::strtoll(string, &ptr, 10));
@@ -323,7 +317,7 @@ void middle_pgsql_t::node_set(osmium::Node const &node)
     if (!m_options->flat_node_file.empty()) {
         m_persistent_cache->set(node.id(), node.location());
     } else {
-        m_db_copy.new_line(m_tables(osmium::item_type::node).copy_target());
+        m_db_copy.new_line(m_tables.nodes().copy_target());
 
         m_db_copy.add_columns(node.id(), node.location().y(),
                               node.location().x());
@@ -351,6 +345,38 @@ std::size_t middle_query_pgsql_t::get_way_node_locations_flatnodes(
     return count;
 }
 
+osmium::Location middle_query_pgsql_t::get_node_location_db(osmid_t id) const
+{
+    auto const res =
+        m_sql_conn.exec_prepared("get_node_list", "{{{}}}"_format(id));
+    if (res.num_tuples() == 0) {
+        return osmium::Location{};
+    }
+
+    return osmium::Location{(int)strtol(res.get_value(0, 1), nullptr, 10),
+                            (int)strtol(res.get_value(0, 2), nullptr, 10)};
+}
+
+osmium::Location
+middle_query_pgsql_t::get_node_location_flatnodes(osmid_t id) const
+{
+    if (id >= 0) {
+        return m_persistent_cache->get(id);
+    }
+    return osmium::Location{};
+}
+
+osmium::Location middle_query_pgsql_t::get_node_location(osmid_t id) const
+{
+    auto const loc = m_cache->get(id);
+    if (loc.valid()) {
+        return loc;
+    }
+
+    return m_persistent_cache ? get_node_location_flatnodes(id)
+                              : get_node_location_db(id);
+}
+
 size_t middle_query_pgsql_t::nodes_get_list(osmium::WayNodeList *nodes) const
 {
     return m_persistent_cache ? get_way_node_locations_flatnodes(nodes)
@@ -364,7 +390,7 @@ void middle_pgsql_t::node_delete(osmid_t osm_id)
     if (!m_options->flat_node_file.empty()) {
         m_persistent_cache->set(osm_id, osmium::Location{});
     } else {
-        m_db_copy.new_line(m_tables(osmium::item_type::node).copy_target());
+        m_db_copy.new_line(m_tables.nodes().copy_target());
         m_db_copy.delete_object(osm_id);
     }
 }
@@ -386,7 +412,7 @@ idlist_t middle_pgsql_t::get_rels_by_way(osmid_t osm_id)
 
 void middle_pgsql_t::way_set(osmium::Way const &way)
 {
-    m_db_copy.new_line(m_tables(osmium::item_type::way).copy_target());
+    m_db_copy.new_line(m_tables.ways().copy_target());
 
     m_db_copy.add_column(way.id());
 
@@ -417,8 +443,8 @@ bool middle_query_pgsql_t::way_get(osmid_t id,
         osmium::builder::WayBuilder builder{*buffer};
         builder.set_id(id);
 
-        pgsql_parse_nodes(res.get_value(0, 0), buffer, builder);
-        pgsql_parse_tags(res.get_value(0, 1), buffer, builder);
+        pgsql_parse_nodes(res.get_value(0, 0), buffer, &builder);
+        pgsql_parse_tags(res.get_value(0, 1), buffer, &builder);
     }
 
     buffer->commit();
@@ -426,62 +452,70 @@ bool middle_query_pgsql_t::way_get(osmid_t id,
     return true;
 }
 
-size_t
-middle_query_pgsql_t::rel_way_members_get(osmium::Relation const &rel,
-                                          rolelist_t *roles,
-                                          osmium::memory::Buffer *buffer) const
+std::size_t
+middle_query_pgsql_t::rel_members_get(osmium::Relation const &rel,
+                                      osmium::memory::Buffer *buffer,
+                                      osmium::osm_entity_bits::type types) const
 {
     assert(buffer);
+    assert((types & osmium::osm_entity_bits::relation) == 0);
 
-    util::string_id_list_t id_list;
+    util::string_id_list_t way_ids;
+    idlist_t node_ids;
 
-    for (auto const &m : rel.members()) {
-        if (m.type() == osmium::item_type::way) {
-            id_list.add(m.ref());
+    for (auto const &member : rel.members()) {
+        if (member.type() == osmium::item_type::node &&
+            (types & osmium::osm_entity_bits::node)) {
+            node_ids.push_back(member.ref());
+        } else if (member.type() == osmium::item_type::way &&
+                   (types & osmium::osm_entity_bits::way)) {
+            way_ids.add(member.ref());
         }
     }
 
-    if (id_list.empty()) {
-        return 0;
+    std::size_t members_found = node_ids.size();
+    if (!node_ids.empty()) {
+        osmium::builder::WayNodeListBuilder builder{*buffer};
+        for (auto const id : node_ids) {
+            builder.add_node_ref(id);
+        }
     }
 
-    auto const res = m_sql_conn.exec_prepared("get_way_list", id_list.get());
-    idlist_t const wayidspg = get_ids_from_result(res);
+    if (!way_ids.empty()) {
+        auto const res =
+            m_sql_conn.exec_prepared("get_way_list", way_ids.get());
+        idlist_t const wayidspg = get_ids_from_result(res);
 
-    // Match the list of ways coming from postgres in a different order
-    //   back to the list of ways given by the caller */
-    size_t outres = 0;
-    for (auto const &m : rel.members()) {
-        if (m.type() != osmium::item_type::way) {
-            continue;
-        }
-        for (int j = 0; j < res.num_tuples(); ++j) {
-            if (m.ref() == wayidspg[j]) {
-                {
+        // Match the list of ways coming from postgres in a different order
+        //   back to the list of ways given by the caller
+        for (auto const &m : rel.members()) {
+            if (m.type() != osmium::item_type::way) {
+                continue;
+            }
+            for (int j = 0; j < res.num_tuples(); ++j) {
+                if (m.ref() == wayidspg[static_cast<std::size_t>(j)]) {
                     osmium::builder::WayBuilder builder{*buffer};
                     builder.set_id(m.ref());
 
-                    pgsql_parse_nodes(res.get_value(j, 1), buffer, builder);
-                    pgsql_parse_tags(res.get_value(j, 2), buffer, builder);
-                }
+                    pgsql_parse_nodes(res.get_value(j, 1), buffer, &builder);
+                    pgsql_parse_tags(res.get_value(j, 2), buffer, &builder);
 
-                buffer->commit();
-                if (roles) {
-                    roles->emplace_back(m.role());
+                    ++members_found;
+                    break;
                 }
-                ++outres;
-                break;
             }
         }
     }
 
-    return outres;
+    buffer->commit();
+
+    return members_found;
 }
 
 void middle_pgsql_t::way_delete(osmid_t osm_id)
 {
     assert(m_options->append);
-    m_db_copy.new_line(m_tables(osmium::item_type::way).copy_target());
+    m_db_copy.new_line(m_tables.ways().copy_target());
     m_db_copy.delete_object(osm_id);
 }
 
@@ -494,7 +528,7 @@ void middle_pgsql_t::relation_set(osmium::Relation const &rel)
         parts[osmium::item_type_to_nwr_index(m.type())].push_back(m.ref());
     }
 
-    m_db_copy.new_line(m_tables(osmium::item_type::relation).copy_target());
+    m_db_copy.new_line(m_tables.relations().copy_target());
 
     // id, way offset, relation offset
     m_db_copy.add_columns(rel.id(), parts[0].size(),
@@ -544,8 +578,8 @@ bool middle_query_pgsql_t::relation_get(osmid_t id,
         osmium::builder::RelationBuilder builder{*buffer};
         builder.set_id(id);
 
-        pgsql_parse_members(res.get_value(0, 0), buffer, builder);
-        pgsql_parse_tags(res.get_value(0, 1), buffer, builder);
+        pgsql_parse_members(res.get_value(0, 0), buffer, &builder);
+        pgsql_parse_tags(res.get_value(0, 1), buffer, &builder);
     }
 
     buffer->commit();
@@ -557,15 +591,15 @@ void middle_pgsql_t::relation_delete(osmid_t osm_id)
 {
     assert(m_options->append);
 
-    m_db_copy.new_line(m_tables(osmium::item_type::relation).copy_target());
+    m_db_copy.new_line(m_tables.relations().copy_target());
     m_db_copy.delete_object(osm_id);
 }
 
 void middle_pgsql_t::after_nodes()
 {
     m_db_copy.sync();
-    if (m_options->flat_node_file.empty()) {
-        auto const &table = m_tables(osmium::item_type::node);
+    if (!m_options->append && m_options->flat_node_file.empty()) {
+        auto const &table = m_tables.nodes();
         analyze_table(m_db_connection, table.schema(), table.name());
     }
 }
@@ -573,24 +607,29 @@ void middle_pgsql_t::after_nodes()
 void middle_pgsql_t::after_ways()
 {
     m_db_copy.sync();
-    auto const &table = m_tables(osmium::item_type::way);
-    analyze_table(m_db_connection, table.schema(), table.name());
+    if (!m_options->append) {
+        auto const &table = m_tables.ways();
+        analyze_table(m_db_connection, table.schema(), table.name());
+    }
 }
 
 void middle_pgsql_t::after_relations()
 {
     m_db_copy.sync();
-    auto const &table = m_tables(osmium::item_type::relation);
-    analyze_table(m_db_connection, table.schema(), table.name());
+    if (!m_options->append) {
+        auto const &table = m_tables.relations();
+        analyze_table(m_db_connection, table.schema(), table.name());
+    }
 
     // release the copy thread and its database connection
     m_copy_thread->finish();
 }
 
 middle_query_pgsql_t::middle_query_pgsql_t(
-    std::string const &conninfo, std::shared_ptr<node_ram_cache> const &cache,
-    std::shared_ptr<node_persistent_cache> const &persistent_cache)
-: m_sql_conn(conninfo), m_cache(cache), m_persistent_cache(persistent_cache)
+    std::string const &conninfo, std::shared_ptr<node_locations_t> cache,
+    std::shared_ptr<node_persistent_cache> persistent_cache)
+: m_sql_conn(conninfo), m_cache(std::move(cache)),
+  m_persistent_cache(std::move(persistent_cache))
 {
     // Disable JIT and parallel workers as they are known to cause
     // problems when accessing the intarrays.
@@ -624,7 +663,7 @@ void middle_pgsql_t::start()
     }
 }
 
-void middle_pgsql_t::stop(thread_pool_t &pool)
+void middle_pgsql_t::stop()
 {
     m_cache.reset();
     if (!m_options->flat_node_file.empty()) {
@@ -640,13 +679,22 @@ void middle_pgsql_t::stop(thread_pool_t &pool)
     } else if (!m_options->append) {
         // Building the indexes takes time, so do it asynchronously.
         for (auto &table : m_tables) {
-            pool.submit(std::bind(&middle_pgsql_t::table_desc::build_index,
-                                  &table,
-                                  m_options->database_options.conninfo()));
+            table.task_set(thread_pool().submit([&]() {
+                table.build_index(m_options->database_options.conninfo());
+            }));
         }
     }
 }
 
+void middle_pgsql_t::wait()
+{
+    for (auto &table : m_tables) {
+        auto const run_time = table.task_wait();
+        log_info("Done postprocessing on table '{}' in {}", table.name(),
+                 util::human_readable_duration(run_time));
+    }
+}
+
 static table_sql sql_for_nodes(bool create_table) noexcept
 {
     table_sql sql{};
@@ -654,15 +702,17 @@ static table_sql sql_for_nodes(bool create_table) noexcept
     sql.name = "{prefix}_nodes";
 
     if (create_table) {
-        sql.create_table = "CREATE {unlogged} TABLE {schema}{prefix}_nodes ("
-                           "  id int8 PRIMARY KEY {using_tablespace},"
-                           "  lat int4 NOT NULL,"
-                           "  lon int4 NOT NULL"
-                           ") {data_tablespace};\n";
+        sql.create_table =
+            "CREATE {unlogged} TABLE {schema}\"{prefix}_nodes\" ("
+            "  id int8 PRIMARY KEY {using_tablespace},"
+            "  lat int4 NOT NULL,"
+            "  lon int4 NOT NULL"
+            ") {data_tablespace};\n";
 
-        sql.prepare_query = "PREPARE get_node_list(int8[]) AS"
-                            "  SELECT id, lon, lat FROM {schema}{prefix}_nodes"
-                            "  WHERE id = ANY($1::int8[]);\n";
+        sql.prepare_query =
+            "PREPARE get_node_list(int8[]) AS"
+            "  SELECT id, lon, lat FROM {schema}\"{prefix}_nodes\""
+            "  WHERE id = ANY($1::int8[]);\n";
     }
 
     return sql;
@@ -675,7 +725,7 @@ static table_sql sql_for_ways(bool has_bucket_index,
 
     sql.name = "{prefix}_ways";
 
-    sql.create_table = "CREATE {unlogged} TABLE {schema}{prefix}_ways ("
+    sql.create_table = "CREATE {unlogged} TABLE {schema}\"{prefix}_ways\" ("
                        "  id int8 PRIMARY KEY {using_tablespace},"
                        "  nodes int8[] NOT NULL,"
                        "  tags text[]"
@@ -683,40 +733,41 @@ static table_sql sql_for_ways(bool has_bucket_index,
 
     sql.prepare_query = "PREPARE get_way(int8) AS"
                         "  SELECT nodes, tags"
-                        "    FROM {schema}{prefix}_ways WHERE id = $1;\n"
+                        "    FROM {schema}\"{prefix}_ways\" WHERE id = $1;\n"
                         "PREPARE get_way_list(int8[]) AS"
                         "  SELECT id, nodes, tags"
-                        "    FROM {schema}{prefix}_ways"
+                        "    FROM {schema}\"{prefix}_ways\""
                         "      WHERE id = ANY($1::int8[]);\n";
 
     if (has_bucket_index) {
         sql.prepare_fw_dep_lookups =
             "PREPARE mark_ways_by_node(int8) AS"
-            "  SELECT id FROM {schema}{prefix}_ways w"
+            "  SELECT id FROM {schema}\"{prefix}_ways\" w"
             "    WHERE $1 = ANY(nodes)"
-            "      AND {schema}{prefix}_index_bucket(w.nodes)"
-            "       && {schema}{prefix}_index_bucket(ARRAY[$1]);\n";
+            "      AND {schema}\"{prefix}_index_bucket\"(w.nodes)"
+            "       && {schema}\"{prefix}_index_bucket\"(ARRAY[$1]);\n";
     } else {
-        sql.prepare_fw_dep_lookups = "PREPARE mark_ways_by_node(int8) AS"
-                                     "  SELECT id FROM {schema}{prefix}_ways"
-                                     "    WHERE nodes && ARRAY[$1];\n";
+        sql.prepare_fw_dep_lookups =
+            "PREPARE mark_ways_by_node(int8) AS"
+            "  SELECT id FROM {schema}\"{prefix}_ways\""
+            "    WHERE nodes && ARRAY[$1];\n";
     }
 
     if (way_node_index_id_shift == 0) {
         sql.create_fw_dep_indexes =
-            "CREATE INDEX ON {schema}{prefix}_ways USING GIN (nodes)"
+            "CREATE INDEX ON {schema}\"{prefix}_ways\" USING GIN (nodes)"
             "  WITH (fastupdate = off) {index_tablespace};\n";
     } else {
         sql.create_fw_dep_indexes =
             "CREATE OR REPLACE FUNCTION"
-            "    {schema}{prefix}_index_bucket(int8[])"
+            "    {schema}\"{prefix}_index_bucket\"(int8[])"
             "  RETURNS int8[] AS $$\n"
             "  SELECT ARRAY(SELECT DISTINCT"
             "    unnest($1) >> {way_node_index_id_shift})\n"
             "$$ LANGUAGE SQL IMMUTABLE;\n"
-            "CREATE INDEX {schema}{prefix}_ways_nodes_bucket_idx"
-            "  ON {schema}{prefix}_ways"
-            "  USING GIN ({schema}{prefix}_index_bucket(nodes))"
+            "CREATE INDEX \"{prefix}_ways_nodes_bucket_idx\""
+            "  ON {schema}\"{prefix}_ways\""
+            "  USING GIN ({schema}\"{prefix}_index_bucket\"(nodes))"
             "  WITH (fastupdate = off) {index_tablespace};\n";
     }
 
@@ -729,7 +780,7 @@ static table_sql sql_for_relations() noexcept
 
     sql.name = "{prefix}_rels";
 
-    sql.create_table = "CREATE {unlogged} TABLE {schema}{prefix}_rels ("
+    sql.create_table = "CREATE {unlogged} TABLE {schema}\"{prefix}_rels\" ("
                        "  id int8 PRIMARY KEY {using_tablespace},"
                        "  way_off int2,"
                        "  rel_off int2,"
@@ -740,20 +791,20 @@ static table_sql sql_for_relations() noexcept
 
     sql.prepare_query = "PREPARE get_rel(int8) AS"
                         "  SELECT members, tags"
-                        "    FROM {schema}{prefix}_rels WHERE id = $1;\n";
+                        "    FROM {schema}\"{prefix}_rels\" WHERE id = $1;\n";
 
     sql.prepare_fw_dep_lookups =
         "PREPARE mark_rels_by_node(int8) AS"
-        "  SELECT id FROM {schema}{prefix}_rels"
+        "  SELECT id FROM {schema}\"{prefix}_rels\""
         "    WHERE parts && ARRAY[$1]"
         "      AND parts[1:way_off] && ARRAY[$1];\n"
         "PREPARE mark_rels_by_way(int8) AS"
-        "  SELECT id FROM {schema}{prefix}_rels"
+        "  SELECT id FROM {schema}\"{prefix}_rels\""
         "    WHERE parts && ARRAY[$1]"
         "      AND parts[way_off+1:rel_off] && ARRAY[$1];\n";
 
     sql.create_fw_dep_indexes =
-        "CREATE INDEX ON {schema}{prefix}_rels USING GIN (parts)"
+        "CREATE INDEX ON {schema}\"{prefix}_rels\" USING GIN (parts)"
         "  WITH (fastupdate = off) {index_tablespace};\n";
 
     return sql;
@@ -769,18 +820,19 @@ static bool check_bucket_index(pg_conn_t *db_connection,
     return res.num_tuples() > 0;
 }
 
-middle_pgsql_t::middle_pgsql_t(options_t const *options)
-: m_options(options),
-  m_cache(new node_ram_cache{options->alloc_chunkwise | ALLOC_LOSSY,
-                             options->cache}),
+middle_pgsql_t::middle_pgsql_t(std::shared_ptr<thread_pool_t> thread_pool,
+                               options_t const *options)
+: middle_t(std::move(thread_pool)), m_options(options),
+  m_cache(std::make_unique<node_locations_t>(
+      static_cast<std::size_t>(options->cache) * 1024UL * 1024UL)),
   m_db_connection(m_options->database_options.conninfo()),
   m_copy_thread(
       std::make_shared<db_copy_thread_t>(options->database_options.conninfo())),
   m_db_copy(m_copy_thread)
 {
     if (!options->flat_node_file.empty()) {
-        m_persistent_cache.reset(new node_persistent_cache{
-            options->flat_node_file, options->droptemp});
+        m_persistent_cache = std::make_shared<node_persistent_cache>(
+            options->flat_node_file, options->droptemp);
     }
 
     log_debug("Mid: pgsql, cache={}", options->cache);
@@ -793,13 +845,12 @@ middle_pgsql_t::middle_pgsql_t(options_t const *options)
         log_debug("You don't have a bucket index. See manual for details.");
     }
 
-    m_tables(osmium::item_type::node) =
+    m_tables.nodes() =
         table_desc{*options, sql_for_nodes(options->flat_node_file.empty())};
-    m_tables(osmium::item_type::way) =
+    m_tables.ways() =
         table_desc{*options, sql_for_ways(has_bucket_index,
                                           options->way_node_index_id_shift)};
-    m_tables(osmium::item_type::relation) =
-        table_desc{*options, sql_for_relations()};
+    m_tables.relations() = table_desc{*options, sql_for_relations()};
 }
 
 std::shared_ptr<middle_query_t>
@@ -807,9 +858,8 @@ middle_pgsql_t::get_query_instance()
 {
     // NOTE: this is thread safe for use in pending async processing only because
     // during that process they are only read from
-    std::unique_ptr<middle_query_pgsql_t> mid{
-        new middle_query_pgsql_t{m_options->database_options.conninfo(),
-                                 m_cache, m_persistent_cache}};
+    auto mid = std::make_unique<middle_query_pgsql_t>(
+        m_options->database_options.conninfo(), m_cache, m_persistent_cache);
 
     // We use a connection per table to enable the use of COPY
     for (auto &table : m_tables) {
