@@ -34,9 +34,10 @@ class middle_query_pgsql_t : public middle_query_t
 {
 public:
     middle_query_pgsql_t(
-        std::string const &conninfo,
-        std::shared_ptr<node_locations_t> const &cache,
-        std::shared_ptr<node_persistent_cache> const &persistent_cache);
+        std::string const &conninfo, std::shared_ptr<node_locations_t> cache,
+        std::shared_ptr<node_persistent_cache> persistent_cache);
+
+    osmium::Location get_node_location(osmid_t id) const override;
 
     size_t nodes_get_list(osmium::WayNodeList *nodes) const override;
 
@@ -52,6 +53,8 @@ public:
     void exec_sql(std::string const &sql_cmd) const;
 
 private:
+    osmium::Location get_node_location_flatnodes(osmid_t id) const;
+    osmium::Location get_node_location_db(osmid_t id) const;
     std::size_t get_way_node_locations_flatnodes(osmium::WayNodeList *nodes) const;
     std::size_t get_way_node_locations_db(osmium::WayNodeList *nodes) const;
 
