@@ -184,6 +184,11 @@ void middle_ram_t::relation(osmium::Relation const &relation)
     }
 }
 
+osmium::Location middle_ram_t::get_node_location(osmid_t id) const
+{
+    return m_node_locations.get(id);
+}
+
 std::size_t middle_ram_t::nodes_get_list(osmium::WayNodeList *nodes) const
 {
     assert(nodes);
@@ -259,6 +264,13 @@ middle_ram_t::rel_members_get(osmium::Relation const &rel,
                     buffer->commit();
                     ++count;
                 }
+            } else {
+                {
+                    osmium::builder::NodeBuilder builder{*buffer};
+                    builder.set_id(member.ref());
+                }
+                buffer->commit();
+                ++count;
             }
             break;
         case osmium::item_type::way:
