@@ -13,7 +13,7 @@
 #include "expire-tiles.hpp"
 #include "flex-table-column.hpp"
 #include "flex-table.hpp"
-#include "osmium-builder.hpp"
+#include "geom.hpp"
 #include "output.hpp"
 
 #include <osmium/index/id_set.hpp>
@@ -153,12 +153,20 @@ public:
 
     void merge_expire_trees(output_t *other) override;
 
+    int app_as_point();
+    int app_as_linestring();
+    int app_as_polygon();
+    int app_as_multilinestring();
+    int app_as_multipolygon();
+    int app_as_geometrycollection();
+
     int app_define_table();
-    int app_mark_way();
     int app_get_bbox();
+    int app_mark_way();
 
     int table_tostring();
     int table_add_row();
+    int table_insert();
     int table_name();
     int table_schema();
     int table_cluster();
@@ -166,7 +174,7 @@ public:
 
 private:
     void init_clone();
-    void select_relation_members(osmium::Relation const &relation);
+    void select_relation_members();
 
     /**
      * Call a Lua function that was "prepared" earlier with the OSMObject
@@ -187,25 +195,30 @@ private:
 
     flex_table_t const &get_table_from_param();
 
+    void check_context_and_state(char const *name, char const *context,
+                                 bool condition);
+
     void write_column(db_copy_mgr_t<db_deleter_by_type_and_id_t> *copy_mgr,
                       flex_table_column_t const &column);
+
     void write_row(table_connection_t *table_connection,
                    osmium::item_type id_type, osmid_t id,
-                   std::string const &geom, int srid);
+                   geom::geometry_t const &geom, int srid);
 
-    geom::osmium_builder_t::wkbs_t
-    run_transform(geom::osmium_builder_t *builder,
-                  geom_transform_t const *transform,
-                  table_column_type target_geom_type, osmium::Node const &node);
+    osmium::OSMObject const &
+    check_and_get_context_object(flex_table_t const &table);
 
-    geom::osmium_builder_t::wkbs_t
-    run_transform(geom::osmium_builder_t *builder,
-                  geom_transform_t const *transform,
-                  table_column_type target_geom_type, osmium::Way const &way);
+    geom::geometry_t run_transform(reprojection const &proj,
+                                   geom_transform_t const *transform,
+                                   osmium::Node const &node);
 
-    geom::osmium_builder_t::wkbs_t run_transform(
-        geom::osmium_builder_t *builder, geom_transform_t const *transform,
-        table_column_type target_geom_type, osmium::Relation const &relation);
+    geom::geometry_t run_transform(reprojection const &proj,
+                                   geom_transform_t const *transform,
+                                   osmium::Way const &way);
+
+    geom::geometry_t run_transform(reprojection const &proj,
+                                   geom_transform_t const *transform,
+                                   osmium::Relation const &relation);
 
     template <typename OBJECT>
     void add_row(table_connection_t *table_connection, OBJECT const &object);
@@ -214,10 +227,59 @@ private:
                            osmium::item_type type, osmid_t osm_id);
     void delete_from_tables(osmium::item_type type, osmid_t osm_id);
 
-    std::size_t get_way_nodes();
-
     lua_State *lua_state() noexcept { return m_lua_state.get(); }
 
+    class way_cache_t
+    {
+    public:
+        bool init(middle_query_t const &middle, osmid_t id);
+        void init(osmium::Way *way);
+        std::size_t add_nodes(middle_query_t const &middle);
+        osmium::Way const &get() const noexcept { return *m_way; }
+
+    private:
+        osmium::memory::Buffer m_buffer{32768,
+                                        osmium::memory::Buffer::auto_grow::yes};
+        osmium::Way *m_way = nullptr;
+        std::size_t m_num_way_nodes = std::numeric_limits<std::size_t>::max();
+
+    }; // way_cache_t
+
+    class relation_cache_t
+    {
+    public:
+        bool init(middle_query_t const &middle, osmid_t id);
+        void init(osmium::Relation const &relation);
+
+        /**
+         * Add members of relation to cache when it is first called.
+         *
+         * \returns True if at least one member was found, false otherwise
+         */
+        bool add_members(middle_query_t const &middle);
+
+        osmium::Relation const &get() const noexcept
+        {
+            return *m_relation;
+        }
+
+        osmium::memory::Buffer const &members_buffer() const noexcept
+        {
+            return m_members_buffer;
+        }
+
+    private:
+        // This buffer is used for the relation only. Members are stored in
+        // m_members_buffer. If we would store more objects in this buffer,
+        // it might autogrow which would invalidate the m_relation pointer.
+        osmium::memory::Buffer m_relation_buffer{
+            1024, osmium::memory::Buffer::auto_grow::yes};
+        osmium::memory::Buffer m_members_buffer{
+            32768, osmium::memory::Buffer::auto_grow::yes};
+        osmium::Relation const *m_relation = nullptr;
+
+    }; // relation_cache_t
+
     std::shared_ptr<std::vector<flex_table_t>> m_tables;
     std::vector<table_connection_t> m_table_connections;
 
@@ -233,19 +295,9 @@ private:
 
     expire_tiles m_expire;
 
-    osmium::memory::Buffer m_buffer;
-
-    // This buffer is used for one relation at a time only. Members of the
-    // relation are stored in m_buffer instead. If we would store more objects
-    // in this buffer, it might autogrow which would invalidate the reference
-    // we take to the relation inside.
-    osmium::memory::Buffer m_rels_buffer;
-
+    way_cache_t m_way_cache;
+    relation_cache_t m_relation_cache;
     osmium::Node const *m_context_node = nullptr;
-    osmium::Way *m_context_way = nullptr;
-    osmium::Relation const *m_context_relation = nullptr;
-
-    std::size_t m_num_way_nodes = std::numeric_limits<std::size_t>::max();
 
     prepared_lua_function_t m_process_node;
     prepared_lua_function_t m_process_way;
