@@ -21,7 +21,7 @@ static testing::db::import_t db;
 // Use a random device with a fixed seed. We don't really care about
 // the quality of random numbers here, we just need to generate valid
 // OSM test data. The fixed seed ensures that the results are reproducible.
-static std::mt19937_64 rng{47382}; // NOLINT(cert-msc32-c)
+static std::mt19937_64 rng{47382}; // NOLINT(cert-msc32-c,cert-msc51-cpp)
 
 class node_opl_t
 {
@@ -214,8 +214,7 @@ public:
         REQUIRE_NOTHROW(db.run_import(opt, opl.c_str()));
     }
 
-    unsigned long obj_count(testing::pg::conn_t const &conn, osmid_t id,
-                            char const *cls)
+    int obj_count(testing::pg::conn_t const &conn, osmid_t id, char const *cls)
     {
         char const tchar = m_opl_factory.type();
         return conn.get_count("place",
@@ -698,6 +697,7 @@ TEST_CASE("Admin levels")
     t.add(9003, "place=city,admin_level=x");
     t.add(9004, "place=city,admin_level=1");
     t.add(9005, "place=city,admin_level=0");
+    t.add(9006, "place=city,admin_level=2.5");
 
     t.import();
 
@@ -708,6 +708,7 @@ TEST_CASE("Admin levels")
     CHECK("15" == t.obj_field(conn, 9003, "place", "admin_level"));
     CHECK("1" == t.obj_field(conn, 9004, "place", "admin_level"));
     CHECK("15" == t.obj_field(conn, 9005, "place", "admin_level"));
+    CHECK("15" == t.obj_field(conn, 9006, "place", "admin_level"));
 }
 
 TEST_CASE("Administrative boundaries with place tags")
